Package: tidyexposomics
Title: Integrated Exposure-Omics Analysis Powered by Tidy Principles
Version: 1.0.0
Authors@R: 
    c(
    person(given = "Jason",
    family  = "Laird",
    email = "jasonlaird5353@gmail.com",
    role = c("aut", "cre"),
    comment = c(ORCID = "0009-0000-5994-2236")
    ),
    person(given = "Thomas",
    family  = "Hartung",
    email = "thartun1@jhu.edu",
    role = c("ctb"),
    comment = c(ORCID = "0000-0003-1359-7689")
    ),
    person(given = "Fenna",
    family  = "Sillé",
    email = "fsille1@jhu.edu",
    role = c("ctb"),
    comment = c(ORCID = "0000-0003-4305-2049")
    ),
    person(given = "Alexandra",
    family  = "Maertens",
    email = "amaerte1@jhu.edu",
    role = c("ctb"),
    comment = c(ORCID = "0000-0002-2077-2011")
    ),
    person("JHU Discovery Award", role = "fnd")
    )
Description: The tidyexposomics package is designed to facilitate the integration of exposure and omics data to identify exposure-omics associations. We structure our commands to fit into the tidyverse framework, where commands are designed to be simplified and intuitive. Here we provide functionality to perform quality control, sample and exposure association analysis, differential abundance analysis, multi-omics integration, and functional enrichment analysis.
License: MIT + file LICENSE
URL: https://bionomad.github.io/tidyexposomics/
BugReports: https://github.com/BioNomad/tidyexposomics/issues
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
biocViews: Software, Transcriptomics, GeneExpression, Epigenetics,
        Proteomics, DifferentialExpression, DifferentialMethylation,
        QualityControl, GraphAndNetwork, MultipleComparison,
        Regression, StatisticalMethod, Visualization, WorkflowStep
Imports: BiocFileCache, broom, cluster, dplyr, DT, factoextra, fenr,
        ggplot2 (>= 3.4.0), ggpubr, ggrepel, Hmisc, httr, igraph,
        jsonlite, limma, MASS, methods, mixOmics, naniar, purrr, readr,
        RGCCA, rlang, S4Vectors, scales, shiny, stats, stringr,
        SummarizedExperiment, tibble, tidybulk, tidyr, utils
Depends: R (>= 4.5.0), MultiAssayExperiment
Suggests: BiocStyle, circlize, curl, densityClust, DiagrammeR,
        dynamicTreeCut, edgeR, forcats, ggh4x, ggnewscale, ggraph,
        ggridges, ggsci, ggvenn, grid, gridExtra, impute, janitor,
        knitr, matrixStats, mice, mirt, missForest, MOFA2, nipalsMCIA,
        openxlsx, patchwork, reticulate, rmarkdown, testthat (>=
        3.0.0), tidygraph, tidyHeatmap, tidytext, tidyverse
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/tidyexposomics
git_branch: RELEASE_3_23
git_last_commit: b47c045
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:11:56 UTC; biocbuild
Author: Jason Laird [aut, cre] (ORCID: <https://orcid.org/0009-0000-5994-2236>),
  Thomas Hartung [ctb] (ORCID: <https://orcid.org/0000-0003-1359-7689>),
  Fenna Sillé [ctb] (ORCID: <https://orcid.org/0000-0003-4305-2049>),
  Alexandra Maertens [ctb] (ORCID:
    <https://orcid.org/0000-0002-2077-2011>),
  JHU Discovery Award [fnd]
Maintainer: Jason Laird <jasonlaird5353@gmail.com>
