# Generated by roxygen2: do not edit by hand

export(build_ont_annot_app)
export(create_exposomicset)
export(download_dataset)
export(extract_omics_exposure_df)
export(extract_results)
export(extract_results_excel)
export(extract_top_factor_features)
export(filter_missing)
export(filter_non_normal)
export(filter_omics)
export(filter_sample_outliers)
export(load_annotation_data)
export(make_example_data)
export(pivot_exp)
export(pivot_feature)
export(pivot_sample)
export(plot_association)
export(plot_circos_correlation)
export(plot_correlation_summary)
export(plot_correlation_tile)
export(plot_enrichment)
export(plot_exposure_impact)
export(plot_exposure_omics_association)
export(plot_exposures)
export(plot_factor_summary)
export(plot_manhattan)
export(plot_missing)
export(plot_network)
export(plot_normality_summary)
export(plot_pca)
export(plot_sample_clusters)
export(plot_sensitivity_summary)
export(plot_top_factor_features)
export(plot_volcano)
export(run_association)
export(run_cluster_samples)
export(run_correlation)
export(run_create_network)
export(run_differential_abundance)
export(run_enrichment)
export(run_exposome_score)
export(run_exposure_impact)
export(run_exposure_omics_association)
export(run_factor_overlap)
export(run_impute_missing)
export(run_multiomics_integration)
export(run_normality_check)
export(run_pca)
export(run_pipeline_summary)
export(run_sensitivity_analysis)
export(run_summarize_exposures)
export(transform_exposure)
import(dplyr)
import(fenr)
import(ggplot2)
import(stringr)
import(tibble)
import(tidyr)
importFrom(BiocFileCache,BiocFileCache)
importFrom(BiocFileCache,bfcrpath)
importFrom(DT,DTOutput)
importFrom(DT,dataTableProxy)
importFrom(DT,datatable)
importFrom(DT,formatStyle)
importFrom(DT,renderDT)
importFrom(DT,replaceData)
importFrom(DT,styleEqual)
importFrom(Hmisc,rcorr)
importFrom(MASS,boxcox)
importFrom(MultiAssayExperiment,colData)
importFrom(MultiAssayExperiment,experiments)
importFrom(MultiAssayExperiment,metadata)
importFrom(RGCCA,rgcca)
importFrom(S4Vectors,DataFrame)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(broom,tidy)
importFrom(cluster,clusGap)
importFrom(cluster,daisy)
importFrom(cluster,diana)
importFrom(cluster,maxSE)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,select_if)
importFrom(dplyr,semi_join)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,sym)
importFrom(dplyr,top_n)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(factoextra,fviz_eig)
importFrom(factoextra,fviz_nbclust)
importFrom(factoextra,hcut)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggpubr,get_palette)
importFrom(ggpubr,rotate_x_text)
importFrom(ggpubr,theme_pubclean)
importFrom(ggpubr,theme_pubr)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(httr,GET)
importFrom(httr,status_code)
importFrom(httr,timeout)
importFrom(igraph,cluster_louvain)
importFrom(igraph,gorder)
importFrom(igraph,graph_from_data_frame)
importFrom(jsonlite,fromJSON)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(methods,as)
importFrom(mixOmics,block.splsda)
importFrom(naniar,impute_mean_all)
importFrom(naniar,impute_median_all)
importFrom(naniar,miss_var_summary)
importFrom(purrr,compact)
importFrom(purrr,discard)
importFrom(purrr,imap)
importFrom(purrr,iwalk)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,pluck)
importFrom(purrr,reduce)
importFrom(readr,parse_number)
importFrom(rlang,.data)
importFrom(rlang,`%||%`)
importFrom(rlang,set_names)
importFrom(rlang,sym)
importFrom(scales,alpha)
importFrom(shiny,HTML)
importFrom(shiny,actionButton)
importFrom(shiny,addResourcePath)
importFrom(shiny,column)
importFrom(shiny,div)
importFrom(shiny,downloadButton)
importFrom(shiny,fileInput)
importFrom(shiny,fluidPage)
importFrom(shiny,fluidRow)
importFrom(shiny,h4)
importFrom(shiny,hr)
importFrom(shiny,img)
importFrom(shiny,modalDialog)
importFrom(shiny,numericInput)
importFrom(shiny,observeEvent)
importFrom(shiny,reactiveValues)
importFrom(shiny,renderUI)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,selectizeInput)
importFrom(shiny,shinyApp)
importFrom(shiny,showModal)
importFrom(shiny,tags)
importFrom(shiny,titlePanel)
importFrom(shiny,uiOutput)
importFrom(shiny,updateSelectizeInput)
importFrom(stats,IQR)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,end)
importFrom(stats,family)
importFrom(stats,glm)
importFrom(stats,hclust)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,qchisq)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,start)
importFrom(stats,var)
importFrom(stringr,str_extract)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_trim)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,enframe)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,separate_rows)
importFrom(tidyr,unnest)
importFrom(tools,md5sum)
importFrom(utils,URLencode)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,tail)
importMethodsFrom(tidybulk,identify_abundant)
importMethodsFrom(tidybulk,pivot_sample)
importMethodsFrom(tidybulk,pivot_transcript)
importMethodsFrom(tidybulk,test_differential_abundance)
